"""
BaseProvider Base Class
"""

import logging

logger = logging.getLogger(__name__)


class BaseProvider:
    """
    Base Provider Class

    This should ideally be an AbstractBaseClass
    """


class ProviderError(Exception):
    """
    General Provider Error
    """


class ProviderSearchError(ProviderError):
    """
    Searching Error
    """



"""
API Searching Configuration
"""

from os import getenv
from typing import Dict, List, Union

from dotenv import load_dotenv

from camply.config.file_config import FileConfig

load_dotenv(FileConfig.DOT_CAMPLY_FILE, override=False)

USER_AGENT_STRINGS: List[str] = [
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2228.0 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.1 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2227.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2226.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.4; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.3; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2225.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/41.0.2224.3 Safari/537.36",
    "Mozilla/5.0 (Windows NT 10.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/40.0.2214.93 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2062.124 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.3; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2049.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 4.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/37.0.2049.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.67 Safari/537.36",
    "Mozilla/5.0 (X11; OpenBSD i386) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1985.125 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/36.0.1944.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.3319.102 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2309.372 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.2117.157 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/35.0.1916.47 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1866.237 Safari/537.36",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.137 Safari/4E423F",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/34.0.1847.116 Safari/537.36 Mozilla/5.0 (iPad; U; CPU OS 3_2 like Mac OS X; en-us) AppleWebKit/531.21.10 (KHTML, like Gecko) Version/4.0.4 Mobile/7B334b Safari/531.21.10",
    "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.517 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1667.0 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1664.3 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/32.0.1664.3 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1650.16 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/31.0.1623.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/30.0.1599.17 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.62 Safari/537.36",
    "Mozilla/5.0 (X11; CrOS i686 4319.74.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.57 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/29.0.1547.2 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1468.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1467.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/28.0.1464.0 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1500.55 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 5.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.93 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.90 Safari/537.36",
    "Mozilla/5.0 (X11; NetBSD) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36",
    "Mozilla/5.0 (X11; CrOS i686 3912.101.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/27.0.1453.116 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1312.60 Safari/537.17",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_8_2) AppleWebKit/537.17 (KHTML, like Gecko) Chrome/24.0.1309.0 Safari/537.17",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.15 (KHTML, like Gecko) Chrome/24.0.1295.0 Safari/537.15",
    "Mozilla/5.0 (Windows NT 6.2; WOW64) AppleWebKit/537.14 (KHTML, like Gecko) Chrome/24.0.1292.0 Safari/537.14",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:40.0) Gecko/20100101 Firefox/40.1",
    "Mozilla/5.0 (Windows NT 6.3; rv:36.0) Gecko/20100101 Firefox/36.0",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10; rv:33.0) Gecko/20100101 Firefox/33.0",
    "Mozilla/5.0 (X11; Linux i586; rv:31.0) Gecko/20100101 Firefox/31.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:31.0) Gecko/20130401 Firefox/31.0",
    "Mozilla/5.0 (Windows NT 5.1; rv:31.0) Gecko/20100101 Firefox/31.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:29.0) Gecko/20120101 Firefox/29.0",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:25.0) Gecko/20100101 Firefox/29.0",
    "Mozilla/5.0 (X11; OpenBSD amd64; rv:28.0) Gecko/20100101 Firefox/28.0",
    "Mozilla/5.0 (X11; Linux x86_64; rv:28.0) Gecko/20100101  Firefox/28.0",
    "Mozilla/5.0 (Windows NT 6.1; rv:27.3) Gecko/20130101 Firefox/27.3",
    "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:27.0) Gecko/20121011 Firefox/27.0",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:25.0) Gecko/20100101 Firefox/25.0",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.6; rv:25.0) Gecko/20100101 Firefox/25.0",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:24.0) Gecko/20100101 Firefox/24.0",
    "Mozilla/5.0 (Windows NT 6.0; WOW64; rv:24.0) Gecko/20100101 Firefox/24.0",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:24.0) Gecko/20100101 Firefox/24.0",
    "Mozilla/5.0 (Windows NT 6.2; rv:22.0) Gecko/20130405 Firefox/23.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:23.0) Gecko/20130406 Firefox/23.0",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:23.0) Gecko/20131011 Firefox/23.0",
    "Mozilla/5.0 (Windows NT 6.2; rv:22.0) Gecko/20130405 Firefox/22.0",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:22.0) Gecko/20130328 Firefox/22.0",
    "Mozilla/5.0 (Windows NT 6.1; rv:22.0) Gecko/20130405 Firefox/22.0",
    "Mozilla/5.0 (Microsoft Windows NT 6.2.9200.0); rv:22.0) Gecko/20130405 Firefox/22.0",
    "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:16.0.1) Gecko/20121011 Firefox/21.0.1",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:16.0.1) Gecko/20121011 Firefox/21.0.1",
    "Mozilla/5.0 (Windows NT 6.2; Win64; x64; rv:21.0.0) Gecko/20121011 Firefox/21.0.0",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:21.0) Gecko/20130331 Firefox/21.0",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:21.0) Gecko/20100101 Firefox/21.0",
    "Mozilla/5.0 (X11; Linux i686; rv:21.0) Gecko/20100101 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:21.0) Gecko/20130514 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.2; rv:21.0) Gecko/20130326 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/20130401 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/20130331 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/20130330 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:21.0) Gecko/20100101 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.1; rv:21.0) Gecko/20130401 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.1; rv:21.0) Gecko/20130328 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.1; rv:21.0) Gecko/20100101 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/20130401 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/20130331 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 5.1; rv:21.0) Gecko/20100101 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 5.0; rv:21.0) Gecko/20100101 Firefox/21.0",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.8; rv:21.0) Gecko/20100101 Firefox/21.0",
    "Mozilla/5.0 (Windows NT 6.2; Win64; x64;) Gecko/20100101 Firefox/20.0",
    "Mozilla/5.0 (Windows x86; rv:19.0) Gecko/20100101 Firefox/19.0",
    "Mozilla/5.0 (Windows NT 6.1; rv:6.0) Gecko/20100101 Firefox/19.0",
    "Mozilla/5.0 (Windows NT 6.1; rv:14.0) Gecko/20100101 Firefox/18.0.1",
    "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:18.0)  Gecko/20100101 Firefox/18.0",
    "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:17.0) Gecko/20100101 Firefox/17.0.6",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.75.14 (KHTML, like Gecko) Version/7.0.3 Safari/7046A194A",
    "Mozilla/5.0 (iPad; CPU OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5355d Safari/8536.25",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_6_8) AppleWebKit/537.13+ (KHTML, like Gecko) Version/5.1.7 Safari/534.57.2",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_3) AppleWebKit/534.55.3 (KHTML, like Gecko) Version/5.1.3 Safari/534.53.10",
    "Mozilla/5.0 (iPad; CPU OS 5_1 like Mac OS X) AppleWebKit/534.46 (KHTML, like Gecko ) Version/5.1 Mobile/9B176 Safari/7534.48.3",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; de-at) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; da-dk) AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; tr-TR) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; ko-KR) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; fr-FR) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; cs-CZ) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Windows; U; Windows NT 6.0; ja-JP) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; PPC Mac OS X 10_5_8; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; zh-cn) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; sv-se) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ko-kr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; ja-jp) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; it-it) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-fr) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; es-es) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-us) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; en-gb) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; de-de) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.4 Safari/533.20.27",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; sv-SE) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; ja-JP) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 6.0; hu-HU) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 6.0; de-DE) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-RU) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; ja-JP) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; it-IT) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US) AppleWebKit/533.20.25 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_7; en-us) AppleWebKit/534.16+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_6; fr-ch) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; de-de) AppleWebKit/534.15+ (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_5; ar) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Android 2.2; Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.3 Safari/533.19.4",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; zh-HK) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
    "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
    "Mozilla/5.0 (Windows; U; Windows NT 6.0; tr-TR) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
    "Mozilla/5.0 (Windows; U; Windows NT 6.0; nb-NO) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
    "Mozilla/5.0 (Windows; U; Windows NT 6.0; fr-FR) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; zh-TW) AppleWebKit/533.19.4 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
    "Mozilla/5.0 (Windows; U; Windows NT 5.1; ru-RU) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_5_8; zh-cn) AppleWebKit/533.18.1 (KHTML, like Gecko) Version/5.0.2 Safari/533.18.5",
]

USER_AGENTS: List[Dict[str, str]] = [
    {"User-Agent": string} for string in USER_AGENT_STRINGS
]

STANDARD_HEADERS: dict = {
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "en-US,en;q=0.9,la;q=0.8",
    "Cache-Control": "max-age=0",
    "Connection": "keep-alive",
    "Upgrade-Insecure-Requests": "1",
}


class RIDBConfig:
    """
    RIDB API Configuration

    https://ridb.recreation.gov/docs
    """

    _camply_ridb_service_account_api_token: bytes = (
        b"YTc0MTY0NzEtMWI1ZC00YTY0LWFkM2QtYTIzM2U3Y2I1YzQ0"
    )
    _api_key: Union[str, bytes] = getenv(
        "RIDB_API_KEY", _camply_ridb_service_account_api_token
    )
    API_KEY = _camply_ridb_service_account_api_token if _api_key == "" else _api_key

    RIDB_SCHEME: str = "https"
    RIDB_NET_LOC: str = "ridb.recreation.gov"
    RIDB_BASE_PATH: str = "api/v1/"

    # FACILITIES_API_PATH FIELDS
    FACILITIES_API_PATH: str = "facilities"
    CAMPGROUND_FACILITY_FIELD_QUALIFIER: str = "Campground"
    # RECREATION AREA FIELDS
    REC_AREA_API_PATH: str = "recareas"
    # CAMPSITE DETAILS
    CAMPSITE_API_PATH: str = "campsites"


class RecreationBookingConfig:
    """
    Variable Storage Class for Recreation.gov Booking API
    """

    API_SCHEME: str = "https"
    API_NET_LOC = "www.recreation.gov"
    API_BASE_PATH: str = "api/camps/availability/campground/"
    API_MONTH_PATH: str = "month"
    API_REFERRERS: dict = {"Referer": "https://www.recreation.gov/"}

    CAMPSITE_UNAVAILABLE_STRINGS: list = [
        "Reserved",
        "Not Available",
        "Not Reservable",
        "Not Reservable Management",
        "Not Available Cutoff",
        "Lottery",
        "Open",
        "NYR",
    ]

    CAMPSITE_LOCATION_LOOP_DEFAULT: str = "Default Loop"
    CAMPSITE_LOCATION_SITE_DEFAULT: str = "Default Site"

    CAMPSITE_BOOKING_URL: str = "https://www.recreation.gov/camping/campsites"

    RATE_LIMITING = (1.01, 1.51)



"""
Push Notifications Template
"""
import datetime
import logging
from abc import ABC, abstractmethod
from typing import Dict, List, Tuple

import requests

from camply.config import CampsiteContainerFields
from camply.containers import AvailableCampsite

logger = logging.getLogger(__name__)


class NotificationError(Exception):
    """
    Notification Exceptions
    """


class BaseNotifications(ABC):
    """
    Base Notifications
    """

    def __init__(self) -> None:
        """
        Instantiate with a Requests Session
        """
        self.session = requests.Session()

    def __repr__(self) -> str:
        """
        String Representation
        """
        return f"<{self.__class__.__name__}>"

    @abstractmethod
    def send_message(self, message: str, **kwargs):
        """
        Send a message

        Parameters
        ----------
        message: str
            Message Text
        **kwargs
            All kwargs passed to underlying notification method
        """
        pass

    @abstractmethod
    def send_campsites(self, campsites: List[AvailableCampsite], **kwargs):
        """
        Send a message with a campsite object

        Parameters
        ----------
        campsites: List[AvailableCampsite]
        """
        pass

    @classmethod
    def format_standard_campsites(
        cls, campsite: AvailableCampsite
    ) -> Tuple[str, Dict[str, str]]:
        """
        Format Standard Message
        """
        fields = dict()
        message_title = " | ".join(
            [
                campsite.recreation_area,
                campsite.facility_name,
                campsite.booking_date.strftime("%Y-%m-%d"),
            ]
        )
        for key, value in campsite.dict().items():
            if key in [
                CampsiteContainerFields.BOOKING_DATE,
                CampsiteContainerFields.BOOKING_END_DATE,
            ]:
                value: datetime.date
                value: str = value.strftime("%Y-%m-%d")
            elif key == CampsiteContainerFields.BOOKING_URL:
                key = "booking_link"
            elif key == CampsiteContainerFields.PERMITTED_EQUIPMENT:
                equipment = (
                    []
                    if campsite.permitted_equipment is None
                    else campsite.permitted_equipment
                )
                value = "\n  - " + "\n  - ".join(
                    set(item.equipment_name for item in equipment)
                )
            if key not in [CampsiteContainerFields.CAMPSITE_ATTRIBUTES]:
                formatted_key = key.replace("_", " ").title()
                fields[formatted_key] = value
        return message_title, fields



"""
Config __init__ file
"""

from .api_config import (
    STANDARD_HEADERS,
    USER_AGENTS,
    RecreationBookingConfig,
    RIDBConfig,
)
from .data_columns import CampsiteContainerFields, DataColumns
from .file_config import FileConfig
from .notification_config import (
    EmailConfig,
    PushbulletConfig,
    PushoverConfig,
    SlackConfig,
    TelegramConfig,
    TwilioConfig,
)
from .search_config import EquipmentOptions, SearchConfig
from .yellowstone_config import YellowstoneConfig

__all__ = [
    "RecreationBookingConfig",
    "RIDBConfig",
    "STANDARD_HEADERS",
    "USER_AGENTS",
    "CampsiteContainerFields",
    "DataColumns",
    "FileConfig",
    "EmailConfig",
    "PushbulletConfig",
    "PushoverConfig",
    "SlackConfig",
    "TelegramConfig",
    "TwilioConfig",
    "SearchConfig",
    "YellowstoneConfig",
    "EquipmentOptions",
]



"""
Base Pydantic Object for Containers
"""

from typing import Any, Set

from pydantic import BaseModel


class CamplyModel(BaseModel):
    """
    Hashable Pydantic Model
    """

    __unhashable__: Set[str] = set()

    def __hash__(self):
        """
        Hash Method for Pydantic BaseModels
        """
        return hash(self.__class__) + hash(
            tuple(
                value
                for key, value in self.__dict__.items()
                if key not in self.__unhashable__
            )
        )

    def __eq__(self, other: Any) -> bool:
        """
        Exclude Unhashable Fields When Evaluating Equality
        """
        if isinstance(other, CamplyModel):
            return self.dict(exclude=self.__unhashable__) == other.dict(
                exclude=other.__unhashable__
            )
        else:
            return self.dict(exclude=self.__unhashable__) == other


############################
# RecDotGov Base Containers
############################


class RecDotGovAttribute(CamplyModel):
    """
    Attribute Object on the Recreation.gov Campsite
    """

    attribute_category: str
    attribute_id: int
    attribute_name: str
    attribute_value: Any


class RecDotGovEquipment(CamplyModel):
    """
    Equipment Object on the Recreation.gov Campsite
    """

    equipment_name: str
    max_length: float



"""
Project Configuration for Yellowstone Variables
"""

from collections import OrderedDict
from os.path import abspath, join
from pathlib import Path


class FileConfig:
    """
    File Path Storage Class
    """

    HOME_PATH = abspath(Path.home())
    DOT_CAMPLY_FILE = join(HOME_PATH, ".camply")
    _file_config_file = Path(abspath(__file__))
    _config_dir = _file_config_file.parent

    CAMPLY_DIRECTORY = _config_dir.parent
    ROOT_DIRECTORY = CAMPLY_DIRECTORY.parent

    DOT_CAMPLY_FIELDS = OrderedDict(
        PUSHOVER_PUSH_USER=dict(default="", notes="Enables Pushover Notifications"),
        PUSHBULLET_API_TOKEN=dict(default="", notes="Enables Pushbullet Notifications"),
        SLACK_WEBHOOK=dict(default="", notes="Enables Slack Notifications"),
        TELEGRAM_BOT_TOKEN=dict(default="", notes="Enables Telegram Notifications"),
        TELEGRAM_CHAT_ID=dict(
            default="", notes="Telegram Notification will be sent here"
        ),
        TWILIO_ACCOUNT_SID=dict(default="", notes="Twilio Account SID"),
        TWILIO_AUTH_TOKEN=dict(default="", notes="Twilio Auth Token"),
        TWILIO_SOURCE_NUMBER=dict(
            default="", notes="Twilio Source number. E.164 format"
        ),
        TWILIO_DEST_NUMBERS=dict(
            default="", notes="Comma-separated list of phone numbers."
        ),
        EMAIL_TO_ADDRESS=dict(
            default="", notes="Email Notifications will be sent here"
        ),
        EMAIL_USERNAME=dict(default="", notes="Email Authorization Login Username"),
        EMAIL_PASSWORD=dict(default="", notes="Email Authorization Login Password"),
        EMAIL_SMTP_SERVER=dict(
            default="smtp.gmail.com", notes="Email Authorization SMTP Server Address"
        ),
        EMAIL_SMTP_PORT=dict(default=465, notes="Email Authorization SMTP Server Port"),
        EMAIL_FROM_ADDRESS=dict(
            default="camply@juftin.com",
            notes="Email Notifications Will Come From this Email",
        ),
        EMAIL_SUBJECT_LINE=dict(
            default="Camply Notification",
            notes="Email Notifications Will Have This Subject Line",
        ),
        PUSHOVER_PUSH_TOKEN=dict(
            default="",
            notes="Pushover Notifications From Your Custom App " "(not required)",
        ),
        RIDB_API_KEY=dict(
            default="", notes="Personal Recreation.gov API Key (not required)"
        ),
    )



"""
Project Configuration for Data Variable Labels
"""


class DataColumns:
    """
    Variable Storage Class
    """

    CAMPSITE_ID_COLUMN: str = "campsite_code"
    BOOKING_DATE_COLUMN: str = "booking_date"
    BOOKING_END_DATE_COLUMN: str = "booking_end_date"
    BOOKING_NIGHTS_COLUMN: str = "booking_nights"
    CAMPSITE_SITE_NAME_COLUMN: str = "campsite_title"
    CAMPSITE_TYPE_COLUMN: str = "campsite_type"
    CAMPSITE_OCCUPANCY_COLUMN: str = "capacity"
    CAMPSITE_USE_TYPE_COLUMN: str = "campsite_type"
    AVAILABILITY_STATUS_COLUMN: str = "Available"
    RECREATION_AREA_COLUMN: str = "recreation_area"
    FACILITY_NAME_COLUMN: str = "facility_name"
    FACILITY_ID_COLUMN: str = "facility_id"
    BOOKING_URL_COLUMN: str = "booking_url"


class CampsiteContainerFields:
    """
    String Variable Storage Class
    """

    CAMPSITE_ID: str = "campsite_id"
    CAMPGROUND_ID: str = "facility_id"
    BOOKING_DATE: str = "booking_date"
    CAMPSITE_GROUP: str = "campsite_group"
    BOOKING_END_DATE: str = "booking_end_date"
    BOOKING_URL: str = "booking_url"

    PERMITTED_EQUIPMENT: str = "permitted_equipment"
    CAMPSITE_ATTRIBUTES: str = "campsite_attributes"



"""
Utils __init__ file
"""

from .api_utils import filter_json, generate_url
from .general_utils import make_list
from .logging_utils import log_camply

__all__ = ["filter_json", "generate_url", "log_camply", "make_list"]



"""
Push Notifications via Slack
"""

import logging
from typing import List

import requests

from camply.config import SlackConfig
from camply.containers import AvailableCampsite
from camply.notifications.base_notifications import BaseNotifications

logger = logging.getLogger(__name__)


class SlackNotifications(BaseNotifications):
    """
    Push Notifications via Slack
    """

    def __init__(self):
        super().__init__()
        self.session.headers.update({"Content-Type": "application/json"})
        if any([SlackConfig.SLACK_WEBHOOK is None, SlackConfig.SLACK_WEBHOOK == ""]):
            warning_message = (
                "Slack is not configured properly. To send Slack messages "
                "make sure to run `camply configure` or set the "
                "proper environment variable: `SLACK_WEBHOOK`."
            )
            logger.error(warning_message)
            raise EnvironmentError(warning_message)

    def send_message(self, message: str, **kwargs) -> requests.Response:
        """
        Send a message via Slack - if environment variables are configured.

        Parameters
        ----------
        message: str
        blocks: List

        Returns
        -------
        requests.Response
        """
        message_blocks = kwargs.pop("blocks", [])
        message_json = {
            "text": message,
        }
        if message_blocks:
            message_json = {
                "blocks": message_blocks,
            }
        logger.debug(message_json)
        response = self.session.post(
            url=SlackConfig.SLACK_WEBHOOK,
            json=message_json,
        )
        try:
            response.raise_for_status()
        except requests.HTTPError as he:
            logger.warning(
                "Notifications weren't able to be sent to Slack. "
                "Your configuration might be incorrect."
            )
            raise ConnectionError(response.text) from he
        return response

    def send_campsites(self, campsites: List[AvailableCampsite], **kwargs):
        """
        Send a message with a campsite object

        Parameters
        ----------
        campsites: AvailableCampsite
        """
        for campsite in campsites:
            message_title, formatted_dict = self.format_standard_campsites(
                campsite=campsite,
            )
            fields = []
            for key, value in formatted_dict.items():
                fields.append(
                    {
                        "type": "mrkdwn",
                        "text": f"*{key}*",
                    }
                )
                if key in ["Permitted Equipment", "Booking Link"]:
                    data_type = "mrkdwn"
                else:
                    data_type = "plain_text"
                fields.append(
                    {
                        "type": data_type,
                        "text": str(value),
                    }
                )

            blocks = []
            blocks.append(
                {
                    "type": "header",
                    "text": {
                        "type": "plain_text",
                        "text": message_title,
                    },
                }
            )
            # Slack only allows 10 fields (k+v) per section
            for chunk in range(0, len(fields) + 1, 10):
                chunk_max = chunk + 10
                blocks.append(
                    {
                        "type": "section",
                        "fields": fields[chunk:chunk_max],
                    }
                )
            self.send_message(
                message=message_title,
                blocks=blocks,
            )



"""
camply __init__ file
"""

from ._version import __application__, __version__
from .config import EquipmentOptions
from .containers import AvailableCampsite, SearchWindow
from .providers import RecreationDotGov, YellowstoneLodging
from .search import SearchRecreationDotGov, SearchYellowstone

__all__ = [
    "__version__",
    "__application__",
    "SearchRecreationDotGov",
    "SearchYellowstone",
    "YellowstoneLodging",
    "RecreationDotGov",
    "SearchWindow",
    "AvailableCampsite",
    "EquipmentOptions",
]



"""
Push Notifications via Pushover
"""

import base64
import logging
from typing import List, Optional

import requests

from camply.config import PushoverConfig
from camply.containers import AvailableCampsite
from camply.notifications.base_notifications import BaseNotifications

logger = logging.getLogger(__name__)


class PushoverNotifications(BaseNotifications, logging.StreamHandler):
    """
    Push Notifications via Pushover + a Logging Handler
    """

    def __init__(self, level: Optional[int] = logging.INFO):
        super().__init__()
        self.session.headers.update(PushoverConfig.API_HEADERS)
        logging.StreamHandler.__init__(self)
        self.setLevel(level=level)
        if any([PushoverConfig.PUSH_USER is None, PushoverConfig.PUSH_USER == ""]):
            warning_message = (
                "Pushover is not configured properly. To send pushover messages "
                "make sure to run `camply configure` or set the "
                "proper environment variables: `PUSHOVER_PUSH_USER`, "
                "`PUSHOVER_PUSH_TOKEN`."
            )
            logger.error(warning_message)
            raise EnvironmentError(warning_message)

    def send_message(self, message: str, **kwargs) -> requests.Response:
        """
        Send a message via Pushover - if environment variables are configured

        Parameters
        ----------
        message: str

        Returns
        -------
        requests.Response
        """
        token = (
            PushoverConfig.PUSH_TOKEN
            if PushoverConfig.PUSH_TOKEN not in [None, ""]
            else base64.b64decode(PushoverConfig.PUSHOVER_DEFAULT_API_TOKEN).decode(
                "utf-8"
            )
        )
        response = self.session.post(
            url=PushoverConfig.PUSHOVER_API_ENDPOINT,
            params=dict(
                token=token, user=PushoverConfig.PUSH_USER, message=message, **kwargs
            ),
        )
        try:
            response.raise_for_status()
        except requests.HTTPError as he:
            logger.warning(
                "Notifications weren't able to be sent to Pushover. "
                "Your configuration might be incorrect."
            )
            raise ConnectionError(response.text) from he
        return response

    def emit(self, record: logging.LogRecord):
        """
        Produce a logging record

        Parameters
        ----------
        record: str
            Message to log
        """
        log_formatted_message = "[{:>10}]: {}".format(
            record.levelname.upper(), record.msg
        )
        title = f"Pushover {record.levelname.title()} Message"
        self.send_message(message=log_formatted_message, title=title)

    def send_campsites(self, campsites: List[AvailableCampsite], **kwargs):
        """
        Send a message with a campsite object

        Parameters
        ----------
        campsites: AvailableCampsite
        """
        for campsite in campsites:
            message_title, formatted_dict = self.format_standard_campsites(
                campsite=campsite,
            )
            fields = []
            for key, value in formatted_dict.items():
                if key == "Booking Link":
                    value = f"<a href='{value}'>{value}</a>"
                fields.append(f"<b>{key}:</b> {value}")
            composed_message = "\n".join(fields)
            self.send_message(message=composed_message, title=message_title, html=1)



"""
Storage Containers for the Application
"""

import datetime
import logging
from typing import List, Optional, Tuple, Union

from camply.containers.base_container import (
    CamplyModel,
    RecDotGovAttribute,
    RecDotGovEquipment,
)

logger = logging.getLogger(__name__)


class SearchWindow(CamplyModel):
    """
    Search Window for Campsite Search
    """

    start_date: datetime.date
    end_date: datetime.date

    def get_date_range(self) -> List[datetime.date]:
        """
        Generate a List of Dates Between two Dates

        Returns
        -------
        List[datetime.date]
        """
        return [
            self.start_date + datetime.timedelta(days=x)
            for x in range((self.end_date - self.start_date).days)
        ]


class AvailableCampsite(CamplyModel):
    """
    Campsite Storage

    This container should be universal regardless of API Provider
    """

    campsite_id: Union[int, str]
    booking_date: datetime.datetime
    booking_end_date: datetime.datetime
    booking_nights: int
    campsite_site_name: str
    campsite_loop_name: str
    campsite_type: str
    campsite_occupancy: Tuple[int, int]
    campsite_use_type: str
    availability_status: str
    recreation_area: str
    recreation_area_id: int
    facility_name: str
    facility_id: Union[int, str]
    booking_url: str

    permitted_equipment: Optional[List[RecDotGovEquipment]]
    campsite_attributes: Optional[List[RecDotGovAttribute]]

    __unhashable__ = {"permitted_equipment", "campsite_attributes"}


class CampgroundFacility(CamplyModel):
    """
    Campground Facility Data Storage
    """

    facility_name: str
    recreation_area: str
    facility_id: Union[int, str]
    recreation_area_id: int


class RecreationArea(CamplyModel):
    """
    Recreation Area Data Storage
    """

    recreation_area: str
    recreation_area_id: int
    recreation_area_location: str



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['camply',
 'camply.config',
 'camply.containers',
 'camply.notifications',
 'camply.providers',
 'camply.providers.recreation_dot_gov',
 'camply.providers.xanterra',
 'camply.search',
 'camply.utils']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.1,<7.0',
 'click>=8.0.1',
 'pandas>=1.3,<1.4',
 'pydantic>=1.2,<2.0',
 'python-dotenv>=0.10.4',
 'pytz>=2019.1',
 'requests>=2.26.0',
 'rich>=10.0.0',
 'tenacity>=5.1.0']

extras_require = \
{'all': ['twilio>=7.14.0'], 'twilio': ['twilio>=7.14.0']}

entry_points = \
{'console_scripts': ['camply = camply.cli:cli']}

setup_kwargs = {
    'name': 'camply',
    'version': '0.7.0',
    'description': 'camply, the campsite finder 🏕',
    'long_description': '<div align="center">\n<a href="https://github.com/juftin/camply">\n  <img src="https://raw.githubusercontent.com/juftin/camply/main/docs/source/_static/camply.svg"\n    width="400" height="400" alt="camply">\n</a>\n</div>\n\n`camply`, the campsite finder ⛺️, is a tool to help you book a campground online. Finding\nreservations at sold out campgrounds can be tough. That\'s where camply comes in. It searches the\nAPIs of booking services like https://recreation.gov (which indexes thousands of campgrounds across\nthe USA) to continuously check for cancellations and availabilities to pop up. Once a campsite\nbecomes available, camply sends you a notification to book your spot!\n\n___________\n___________\n\n[![PyPI](https://img.shields.io/pypi/v/camply?color=blue&label=⛺️camply)](https://github.com/juftin/camply)\n[![PyPI - Python Version](https://img.shields.io/pypi/pyversions/camply)](https://pypi.python.org/pypi/camply/)\n[![Docker Image Version](https://img.shields.io/docker/v/juftin/camply?color=blue&label=docker&logo=docker)](https://hub.docker.com/r/juftin/camply)\n[![Testing Status](https://github.com/juftin/camply/actions/workflows/tests.yaml/badge.svg?branch=main)](https://github.com/juftin/camply/actions/workflows/tests.yaml)\n[![GitHub License](https://img.shields.io/github/license/juftin/camply?color=blue&label=License)](https://github.com/juftin/camply/blob/main/LICENSE)\n[![Black Codestyle](https://img.shields.io/badge/code%20style-black-000000.svg)]()\n\n\n## [Check Out The Docs](https://juftin.com/camply/)\n\n## Table of Contents\n\n- [Installation](#installation)\n    * [PyPI](#pypi)\n    * [Docker](#docker)\n    * [Building Locally](#building-locally)\n- [Command Line Usage](#command-line-usage)\n    * [campsites](#campsites)\n    * [recreation-areas](#recreation-areas)\n    * [campgrounds](#campgrounds)\n    * [configure](#configure)\n    * [Examples](#examples)\n        + [Searching for a Campsite](#searching-for-a-campsite)\n        + [Searching for a Campsite by Campground ID](#searching-for-a-campsite-by-campground-id)\n        + [Searching for a Specific Campsite by ID](#searching-for-a-specific-campsite-by-id)\n        + [Continuously Searching for A Campsite](#continuously-searching-for-a-campsite)\n        + [Continue Looking After The First Match Is Found](#continue-looking-after-the-first-match-is-found)\n        + [Send a Push Notification](#send-a-push-notification)\n        + [Send a Text Message](#send-a-text-message)\n        + [Send a Notification to Different Services](#send-a-notification-to-different-services)\n        + [Look for Weekend Campsite Availabilities](#look-for-weekend-campsite-availabilities)\n        + [Look for Consecutive Nights at the Same Campsite](#look-for-consecutive-nights-at-the-same-campsite)\n        + [Look for a Campsite Inside of Yellowstone](#look-for-a-campsite-inside-of-yellowstone)\n        + [Look for a Campsite Across Multiple Recreation areas](#look-for-a-campsite-across-multiple-recreation-areas)\n        + [Using a YAML Configuration file to search for campsites](#using-a-yaml-configuration-file-to-search-for-campsites)\n        + [Searching for a Campsite That Fits Your Equipment](#searching-for-a-campsite-that-fits-your-equipment)\n        + [Saving the Results of a Search](#saving-the-results-of-a-search)\n        + [Search for Recreation Areas by Query String](#search-for-recreation-areas-by-query-string)\n        + [Look for Specific Campgrounds Within a Recreation Area](#look-for-specific-campgrounds-within-a-recreation-area)\n        + [Look for Specific Campgrounds by Query String](#look-for-specific-campgrounds-by-query-string)\n- [Finding Recreation Areas IDs and Campground IDs To Search Without Using the Command Line](#finding-recreation-areas-ids-and-campground-ids-to-search-without-using-the-command-line)\n- [Object-Oriented Usage (Python)](#object-oriented-usage-python)\n    * [Search for a Recreation.gov Campsite](#search-for-a-recreationgov-campsite)\n    * [Continuously Search for Recreation.gov Campsites](#continuously-search-for-recreationgov-campsites)\n- [Running in Docker](#running-in-docker)\n- [Dependencies](#dependencies)\n\n## Installation\n\n### PyPI\n\nCamply is hosted on PyPI, you can download it directly from tools like\npip and pipx.\n\n```shell\npip install camply\n```\n\nIf you\'re using camply on your command line - it\'s recommended that you use a tool\nlike [pipx](https://github.com/pypa/pipx) to easily manage the application in an isolated\nvirtual environment.\n\n```shell\npipx install camply\n```\n\n[PyPI Link](https://pypi.python.org/pypi/camply/)\n\n### Docker\n\n```shell\ndocker pull juftin/camply\n```\n\n[Docker Hub Link](https://hub.docker.com/r/juftin/camply)\n\n**_see [Running in Docker](#running-in-docker) below._\n\n## Command Line Usage\n\nWhen installed, `camply`\'s command line utility can be invoked with the command, `camply`. The CLI\ntool accepts one of four sub-arguments: `campsites`, `recreation-areas`, `campgrounds`,\nand `configure`.\n\n```text\n❯ camply --help\n\nUsage: camply [OPTIONS] COMMAND [ARGS]...\n\n  Welcome to camply, the campsite finder.\n\n  Finding reservations at sold out campgrounds can be tough. That\'s where\n  camply comes in. It searches the APIs of booking services like\n  https://recreation.gov (which indexes thousands of campgrounds across the\n  USA) to continuously check for cancellations and availabilities to pop up.\n  Once a campsite becomes available, camply sends you a notification to book\n  your spot!\n\n  visit the camply documentation at https://github.com/juftin/camply\n\nOptions:\n  --version             Show the version and exit.\n  --provider TEXT       Camping Search Provider. Options available are\n                        \'Yellowstone\' and \'RecreationDotGov\'. Defaults to\n                        \'RecreationDotGov\', not case-sensitive.\n  --debug / --no-debug  Enable extra debugging output\n  --help                Show this message and exit.\n\nCommands:\n  campgrounds       Search for Campgrounds (inside of Recreation Areas)...\n  campsites         Find available Campsites using search criteria\n  configure         Set up camply configuration file with an interactive...\n  recreation-areas  Search for Recreation Areas and list them\n```\n\n### campsites\n\nSearch for a campsite within camply. Campsites are returned based on the search criteria provided.\nCampsites contain properties like booking date, site type (tent, RV, cabin, etc), capacity, price,\nand a link to make the booking. Required parameters include `--start-date`, `--end-date`,\n`--rec-area` / `--campground` / `--campsite`. Constant searching functionality can be enabled with\n`--continuous` and notifications via Email, Pushover, Pushbullet, and Telegram can be enabled using\n`--notifications`.\n\n#### Arguments\n\n* `--rec-area`: `RECREATION_AREA_ID`\n    + Add Recreation Areas (comprised of campgrounds) by ID.\n      [**_example_](#searching-for-a-campsite)\n* `--campground`: `CAMPGROUND_ID`\n    + Add individual Campgrounds by ID.\n      [**_example_](#searching-for-a-campsite-by-campground-id)\n* `--campsite`: `CAMPSITE_ID`\n    + Add individual Campsites by ID.\n      [**_example_](#searching-for-a-specific-campsite-by-id)\n* `--start-date`: `START_DATE`\n    + `YYYY-MM-DD`: Start of Search window. You will be arriving this day.\n      [**_example_](#searching-for-a-campsite)\n* `--end-date`: `END_DATE`\n    + `YYYY-MM-DD`: End of Search window. You will be checking out this day..\n      [**_example_](#searching-for-a-campsite)\n* `--weekends`\n    + Only search for weekend bookings (Fri/Sat nights).\n      [**_example_](#look-for-weekend-campsite-availabilities)\n* `--nights`\n    + Search for campsite stays with consecutive nights. Defaults to 1 which returns all campsites\n      found.\n      [**_example_](#look-for-consecutive-nights-at-the-same-campsite)\n* `--provider`: `PROVIDER`\n    + Camping Search Provider. Options available are \'Yellowstone\' and \'RecreationDotGov\'. Defaults\n      to \'RecreationDotGov\', not case-sensitive.\n      [**_example_](#look-for-a-campsite-inside-of-yellowstone)\n* `--continuous`\n    + Continuously check for a campsite to become available, and quit once at least one campsite is\n      found.\n      [**_example_](#continuously-searching-for-a-campsite)\n* `--search-forever`\n    + If `--continuous` is activated, this method continues to search after the first availability\n      has been found. The one caveat is that it will never notify about the same identical campsite\n      for the same booking date.\n      [**_example_](#continue-looking-after-the-first-match-is-found)\n* `--notifications`: `NOTIFICATIONS`\n    + If `--continuous` is activated, types of notifications to receive. Options available\n      are`email`, `pushover`, `pushbullet`, `telegram`, or `silent`. Defaults to `silent` - which just logs\n      messages to console.\n      [**_example_](#send-a-push-notification)\n* `--equipment`\n    + Search for campsites compatible with your camping equipment. This argument accepts two\n      options, the equipment name and its length If you don\'t want to filter based on length provide\n      a length of 0. Accepted equipment names include `Tent`, `RV`. `Trailer`, `Vehicle` and are\n      not case-sensitive.\n      [**_example_](#searching-for-a-campsite-that-fits-your-equipment)\n* `--notify-first-try`\n    + If `--continuous` is activated, whether to send all non-silent notifications if more than 5\n      matching campsites are found on the first try. Defaults to false which only sends the first5.\n      [**_example_](#continuously-searching-for-a-campsite)\n* `--polling-interval`: `POLLING_INTERVAL`\n    + If `--continuous` is activated, how often to wait in between checks (in minutes). Defaults to\n      10, cannot be less than 5.\n      [**_example_](#look-for-weekend-campsite-availabilities)\n* `--yaml-config`\n    + Rather than provide arguments to the command line utility, instead pass a file path to a YAML\n      configuration file. See the documentation for more information on how to structure your\n      configuration file.\n      [**_example_](#using-a-yaml-configuration-file-to-search-for-campsites)\n\n```shell\ncamply campsites \\\n    --rec-area 2725 \\\n    --start-date 2023-07-10 \\\n    --end-date 2023-07-18\n```\n\n### recreation-areas\n\nSearch for Recreation Areas and their IDs. Recreation Areas are places like National Parks and\nNational Forests that can contain one or many campgrounds.\n\n#### Arguments:\n\n* `--search` `SEARCH`\n    + Search for Campgrounds or Recreation Areas by search string.\n* `--state` `STATE`\n    + Filter by US state code.\n\n```shell\ncamply recreation-areas --search "Yosemite National Park"\n```\n\n**_see the [examples](#search-for-recreation-areas-by-query-string) for more information_\n\n### campgrounds\n\nSearch for Campgrounds and their IDs. Campgrounds are facilities inside of Recreation Areas that\ncontain campsites. Most \'campgrounds\' are areas made up of multiple campsites, others are facilities\nlike fire towers or cabins that might only contain a single \'campsite\' to book.\n\n#### Arguments:\n\n* `--search` `SEARCH`\n    + Search for Campgrounds or Recreation Areas by search string.\n* `--state` `STATE`\n    + Filter by US state code.\n* `--rec-area`: `RECREATION_AREA_ID`\n    + Add Recreation Areas (comprised of campgrounds) by ID.\n* `--campground`: `CAMPGROUND_ID`\n    + Add individual Campgrounds by ID.\n\n```shell\ncamply campgrounds --search "Fire Tower Lookout" --state CA\n```\n\n**_see the [examples](#look-for-specific-campgrounds-by-query-string) for more information_\n\n### configure\n\nSet up `camply` configuration file with an interactive console\n\nIn order to send notifications through `camply` you must set up some authorization values. Whether\nyou need to set up [Pushover notifications](https://pushover.net)\n, [PushBullet](https://www.pushbullet.com/#settings/account), [Telegram](https://core.telegram.org/bots),\nor Email messages, everything can be done through the `configure` command. The end result is a file called\n[`.camply`](docs/examples/example.camply) in your home folder. See\nthe [Running in Docker](#running-in-docker) section to see how you can use environment variables\ninstead of a config file.\n\n```shell\ncamply configure\n```\n\n### Examples\n\nRead through the examples below to get a better understanding of `camply`, its features, and the\nfunctionality of the different arguments provided to the CLI.\n\n#### Searching for a Campsite\n\nThe below search looks for campsites inside of Recreation Area ID #2725 (Glacier National Park)\nbetween 2023-07-10 and 2023-07-17. The search will be performed once and any results will be logged\nto the console. camply searches for campsites inside of search windows in increments of one night.\n`--start-date` and `--end-date` define the bounds of the search window, you will be leaving the day\nafter `--end-date`.\n\n```shell\ncamply campsites \\\n    --rec-area 2725 \\\n    --start-date 2023-07-10 \\\n    --end-date 2023-07-18\n```\n\n#### Searching for a Campsite by Campground ID\n\nThe below search looks for across three campgrounds (all inside Glacier National Park)\nbetween 2023-07-10 and 2023-07-17. Multiple Campgrounds (and Recreation Areas too) can be found by\nsupplying the arguments more than once.\n\n```shell\ncamply campsites \\\n    --campground 232493 \\\n    --campground 251869 \\\n    --campground 232492 \\\n    --start-date 2023-07-10 \\\n    --end-date 2023-07-18\n```\n\n#### Searching for a Specific Campsite by ID\n\nSometimes you have a favorite campsite inside your favorite campground. To search for just a\nspecific campsite (and not just all campsites within a campground) you can give its ID to `camply`\nwith the `--campsite` argument. For example, site `R035` in Many Glacier Campground, MT is close to\na trailhead. Its URL\nis [https://www.recreation.gov/camping/campsites/98363](https://www.recreation.gov/camping/campsites/98363)\n, here we can see that it\'s ID is `98363`. You can search for one or many campsites by ID by\nsupplying the `--campsite` argument. You can provide the `--campsite` argument once or multiple\ntimes to search for different campsites. Note, `--campsite` arguments override any `--rec-area`\nor `--campground` parameters provided.\n\n```shell\ncamply campsites \\\n    --campsite 98363 \\\n    --start-date 2023-07-10 \\\n    --end-date 2023-07-18\n```\n\n#### Continuously Searching for A Campsite\n\nSometimes you want to look for campgrounds until an eventual match is found. The below snippet will\nsearch for matching campsites until it finds a match. It also sends a notification via `pushover`\nonce matches are found. Alternate notification methods are `email`, `pushbullet`, `telegram`, and `silent` (\ndefault).\n\n__Important Note__: When `camply` is told to run `--continuous` with non-silent notifications set up\nand it finds more than 5 matching campsites on the first try, it will only send notifications for\nthe first 5 campsites. This is to prevent thousands of campsites flooding your notifications. It\'s\nalways encouraged to perform an initial online search before setting up a `camply` search. To bypass\nthis behavior and send all notifications, pass the `--notify-first-try` argument.\n\n```shell\ncamply campsites \\\n    --rec-area 2725 \\\n    --start-date 2023-07-12 \\\n    --end-date 2023-07-13 \\\n    --continuous \\\n    --notifications pushover \\\n    --notify-first-try\n```\n\n#### Continue Looking After The First Match Is Found\n\nSometimes you want to search for all possible matches up until your arrival date. No problem. Add\nthe `--search-forever` and `camply` won\'t stop sending notifications after the first match is found.\nOne important note, `camply` will save and store all previous notifications when `--search-forever`\nis enabled, so it won\'t notify you about the exact same campsite availability twice. This can be\nproblematic when certain campsites become available more than once.\n\n```shell\ncamply campsites \\\n    --rec-area 2725 \\\n    --start-date 2023-07-01 \\\n    --end-date 2023-08-01 \\\n    --continuous \\\n    --notifications pushover \\\n    --search-forever\n```\n\n#### Send a Push Notification\n\ncamply supports notifications via `Pushbullet`, `Pushover`, `Telegram`, and `Email`. Pushbullet is a great\noption because it\'s\na [free and easy service to sign up for](https://www.pushbullet.com/#settings/account)\nand it supports notifications across different devices and operating systems. Similar to `Pushover`,\n`Pushbullet` requires that you create an account and an API token, and share that token with camply\nthrough a [configuration file](docs/examples/example.camply) (via the `camply configure`\ncommand) or though environment variables (`PUSHBULLET_API_TOKEN`).\n\n```shell\ncamply campsites \\\n    --rec-area 2991 \\\n    --start-date 2023-09-10 \\\n    --end-date 2023-09-21 \\\n    --continuous \\\n    --notifications pushbullet\n```\n\n#### Send a Text Message\n\nIf you want to sign up for a [Twilio](https://www.twilio.com/try-twilio) account, camply also supports\nsending text messages via SMS. You can set up your Twilio configuration via `camply configure`. You will need\nto set the following config values for Twilio: `TWILIO_ACCOUNT_SID`, `TWILIO_AUTH_TOKEN`,\n`TWILIO_SOURCE_NUMBER`, `TWILIO_DEST_NUMBERS`.\n\nSending text messages via Twilio also requires the `twilio` extras:\n\n```\npip install camply[twilio]\n```\n\n```shell\ncamply campsites \\\n    --rec-area 2991 \\\n    --start-date 2023-09-10 \\\n    --end-date 2023-09-21 \\\n    --continuous \\\n    --notifications twilio\n```\n\n#### Send a Notification to Different Services\n\ncamply supports notifications from different providers. To send notifications to multiple providers\nyou can pass the --notifications parameter multiple times. YAML config entries also\naccept an array as well.\n\n```shell\ncamply campsites \\\n    --rec-area 2991 \\\n    --start-date 2023-09-10 \\\n    --end-date 2023-09-21 \\\n    --continuous \\\n    --notifications email \\\n    --notifications pushover\n```\n\n#### Look for Weekend Campsite Availabilities\n\nThis below search looks across larger periods of time, but only if a campground is available to book\non a Friday or Saturday night (`--weekends`). It also uses the `--polling-interval` argument which\nchecks every 5 minutes instead of the default 10 minutes.\n\n```shell\ncamply campsites \\\n    --rec-area 2991 \\\n    --start-date 2023-05-01 \\\n    --end-date 2023-08-01 \\\n    --weekends \\\n    --continuous \\\n    --notifications email \\\n    --polling-interval 5\n```\n\n#### Look for Consecutive Nights at the Same Campsite\n\nA lot of times you need to search for consecutive nights at the same campsite. By default, any and\nall campsites with a single nights booking are returned by camply. To search for campsites with\nconsecutive night stays, pass the `--nights` argument.\n\nNote, the `--nights` argument handles issues with improper search parameters. For example, if you\nset the `--weekends` parameter the maximum number of consecutive nights possible is 2. If you supply\nmore than this your `--nights` parameter will be overwritten to 2.\n\n```shell\ncamply campsites \\\n    --rec-area 2991 \\\n    --start-date 2023-05-01 \\\n    --end-date 2023-08-01 \\\n    --nights 4\n```\n\n#### Look for a Campsite Inside of Yellowstone\n\nYellowstone doesn\'t use https://recreation.gov to manage its campgrounds, instead it uses its own\nproprietary system. In order to search the Yellowstone API for campsites, make sure to pass\nthe `--provider "yellowstone"` argument. This flag disables `--rec-area` argument.\n\nTo learn more about using `camply` to find campsites at Yellowstone, check out\nthis [discussion](https://github.com/juftin/camply/discussions/15#discussioncomment-783657).\n\n```shell\ncamply campsites \\\n    --provider yellowstone \\\n    --start-date 2023-07-09 \\\n    --end-date 2023-07-17 \\\n    --continuous\n```\n\n#### Look for a Campsite Across Multiple Recreation areas\n\nYou don\'t have to confine your search to a single Recreation or Campground ID. Adding multiple\narguments to the command line will search across multiple IDs. Keep in mind that any `--campground`\narguments will overwrite all `--rec-area` arguments.\n\n```shell\ncamply campsites \\\n    --rec-area 2991 \\\n    --rec-area 1074 \\\n    --start-date 2023-07-09 \\\n    --end-date 2023-07-17 \\\n    --nights 5\n```\n\n#### Using a YAML Configuration file to search for campsites\n\nSometimes, using a YAML configuration file is easier to manage all of your search options. See the\nbelow [YAML example file](docs/examples/example_search.yaml) and corresponding camply command:\n\n```yaml\nprovider:         RecreationDotGov  # RecreationDotGov IF NOT PROVIDED\nrecreation_area: # (LIST OR SINGLE ENTRY)\n  - 2991  # Yosemite National Park, CA (All Campgrounds)\n  - 1074  # Sierra National Forest, CA (All Campgrounds)\ncampgrounds:      null  # ENTIRE FIELD CAN BE OMITTED IF NOT USED - (LIST OR SINGLE ENTRY)\ncampsites:        null   # OVERRIDES CAMPGROUNDS / RECREATION AREA - (LIST OR SINGLE ENTRY)\nstart_date:       2023-09-12  # YYYY-MM-DD\nend_date:         2023-09-13  # YYYY-MM-DD\nweekends:         false  # FALSE BY DEFAULT\nnights:           1  # 1 BY DEFAULT\ncontinuous:       true  # DEFAULTS TO TRUE\npolling_interval: 5  # DEFAULTS TO 10 , CAN\'T BE LESS THAN 5\nnotifications:    email  # (silent, email, pushover, pushbullet, and telegram), DEFAULTS TO `silent`\nsearch_forever:   true  # FALSE BY DEFAULT\nnotify_first_try: false  # FALSE BY DEFAULT\nequipment:        null # Array of Equipment Search Lists - DEFAULTS TO `null`\noffline_search: false    # FALSE BY DEFAULT\noffline_search_path: camply_campsites.json # Defaults to `camply_campsites.json`\n```\n\n```shell\ncamply campsites --yaml-config example_search.yaml\n```\n\n#### Searching for a Campsite That Fits Your Equipment\n\nCamply can help you filter campsites to fit your specific equipment, like a Trailer or an RV.\nMost likely, you care that the campsite fits the length of your RV so you can specify that as well.\nTo search for specific equipment and its length provide the `--equipment` option with two arguments,\nthe equipment name and the equipment length. If you don\'t want to filter based on length provide a\nlength of zero. If you provide multiple `--equipment` options, sites matching any of your search\nequipment will be returned.\n\nCurrent supported equipment names are `Vehicle`, `Tent`, `RV`, and `Trailer`. Be careful when\nfiltering on `Tents` and `Vehicle` length, sometimes Recreation.gov doesn\'t provide that information,\nit is safer to set length to 0.\n\n```shell\ncamply campsites \\\n    --rec-area 2991 \\\n    --start-date 2023-07-09 \\\n    --end-date 2023-07-17 \\\n    --nights 5 \\\n    --equipment RV 25\n```\n\nHere\'s what the above search would look like on a YAML Config:\n\n```yaml\nrecreation_area:\n    - 2991\nstart_date:       2023-07-09\nend_date:         2023-07-17\nnights:           5\nequipment:\n    - [RV, 25]\n```\n\nFinally, here\'s a search that accommodates trailers of all lengths:\n\n```shell\ncamply campsites \\\n    --rec-area 2991 \\\n    --start-date 2023-07-09 \\\n    --end-date 2023-07-17 \\\n    --nights 5 \\\n    --equipment Trailer 0\n```\n\n#### Saving the Results of a Search\n\nIn some cases, you might want to save all the campsites found during one\nsearch and load them into a new search, so you don\'t receive duplicate notifications.\nThis can be achieved by passing the `--offline-search` flag. By default, camply will save\nthe results in a file called `camply_campsites.json`.\n\nOptionally, you can also path the `--offline-search-path` flag to specify a certain file\npath to save the results as. When a file path with a `.json` extension is passed\ncamply will export the results as a JSON file. When the `.pkl` or `.pickle` extension is\nused, camply will use a serialized Pickle file.\n\n```shell\ncamply \\\n  campsites \\\n  --campground 232064 \\\n  --start-date 2023-09-01 \\\n  --end-date 2023-10-01 \\\n  --continuous \\\n  --offline-search\n```\n\n```shell\ncamply \\\n  campsites \\\n  --campground 232064 \\\n  --start-date 2023-09-01 \\\n  --end-date 2023-10-01 \\\n  --continuous \\\n  --offline-search \\\n  --offline-search-path campsites.pkl\n```\n\n#### Search for Recreation Areas by Query String\n\nJust need to find what your local Recreation Area ID number is? This simple command allows you to\nsearch and list recreation areas. It accepts `--search` and `--state` arguments.\n\n```shell\ncamply recreation-areas --search "Yosemite National Park"\n```\n\n#### Look for Specific Campgrounds Within a Recreation Area\n\nNeed to get even more specific and search for a particular campground? This search lists campgrounds\nattached to a recreation area id `--rec-area`. It also accepts `--search` and `--state`\narguments.\n\n```shell\ncamply campgrounds --rec-area 2991\n```\n\n#### Look for Specific Campgrounds by Query String\n\nThe below search looks for Fire Lookout Towers to stay in inside of California.\n\n```shell\ncamply campgrounds --search "Fire Tower Lookout" --state CA\n```\n\n## Finding Recreation Areas IDs and Campground IDs To Search Without Using the Command Line\n\nYou can uncover campground and recreation area IDs just by using the https://recreation.gov search\nfunctionality. Use the below example for a campground within Glacier National Park.\n\nFirst, perform your search on https://recreation.gov.\n\n<div>\n<img src="https://raw.githubusercontent.com/juftin/camply/main/docs/source/_static/recreation_dot_gov.png"\n    width="500" alt="recreation_dot_gov search">\n</div>\n\nThe above search will take you to a URL like this:\nhttps://www.recreation.gov/search?q=Glacier%20National%20Park&entity_id=2725&entity_type=recarea.\nTaking a closer look at the URL components you can see that Glacier National Park has the Recreation\nArea ID #2725.\n\nSearching deeper into campgrounds inside of Glacier National Park you might find Fish Creek\nCampground at a URL like https://www.recreation.gov/camping/campgrounds/232493. Here, we can see\nthat this campground has a Campground ID of #232493.\n\n## Object-Oriented Usage (Python)\n\n### Search for a Recreation.gov Campsite\n\n```python\nfrom datetime import datetime\nimport logging\nfrom typing import List\n\nfrom camply.containers import AvailableCampsite, SearchWindow\nfrom camply.search import SearchRecreationDotGov\n\nlogging.basicConfig(format="%(asctime)s [%(levelname)8s]: %(message)s",\n                    level=logging.INFO)\n\nmonth_of_june = SearchWindow(start_date=datetime(year=2022, month=6, day=1),\n                             end_date=datetime(year=2022, month=6, day=30))\ncamping_finder = SearchRecreationDotGov(search_window=month_of_june,\n                                        recreation_area=2725,  # Glacier Ntl Park\n                                        weekends_only=False,\n                                        nights=1)\nmatches: List[AvailableCampsite] = camping_finder.get_matching_campsites(log=True, verbose=True,\n                                                                         continuous=False)\n```\n\nThe above script returns a list of any matching `AvailableCampsite` pydantic objects:\n\n```python\n[\n    AvailableCampsite(campsite_id="5391",\n                      booking_date=datetime.datetime(2022, 6, 13, 0, 0),\n                      campsite_site_name="B37",\n                      campsite_loop_name="Loop B",\n                      campsite_type="STANDARD NONELECTRIC",\n                      campsite_occupancy=(0, 8),\n                      campsite_use_type="Overnight",\n                      availability_status="Available",\n                      recreation_area="Glacier National Park, MT",\n                      recreation_area_id="2725",\n                      facility_name="Fish Creek Campground",\n                      facility_id="232493",\n                      booking_url="https://www.recreation.gov/camping/campsites/5391")\n]\n```\n\n### Continuously Search for Recreation.gov Campsites\n\nYou\'ll notice that the `get_matching_campsites` function takes accepts parameter values very similar\nto the commandline arguments.\n\n```python\nfrom datetime import datetime\nimport logging\n\nfrom camply.containers import SearchWindow\nfrom camply.search import SearchRecreationDotGov\n\nlogging.basicConfig(format="%(asctime)s [%(levelname)8s]: %(message)s",\n                    level=logging.INFO)\n\nmonth_of_june = SearchWindow(start_date=datetime(year=2022, month=6, day=1),\n                             end_date=datetime(year=2022, month=6, day=30))\ncamping_finder = SearchRecreationDotGov(search_window=month_of_june,\n                                        recreation_area=[2991, 1074],  # Multiple Rec Areas\n                                        weekends_only=False,\n                                        nights=3)\ncamping_finder.get_matching_campsites(log=True, verbose=True,\n                                      continuous=True,\n                                      polling_interval=5,\n                                      notification_provider="pushover",\n                                      search_forever=True,\n                                      notify_first_try=False)\n```\n\n## Running in Docker\n\nHere\'s an example of a detached container searching in the background (notice the `-d` flag, the\ncontainer will run detached).\n\n```shell\ndocker run -d \\\n  --name camply-detached-example \\\n  --env PUSHOVER_PUSH_TOKEN=${PUSHOVER_PUSH_TOKEN} \\\n  --env PUSHOVER_PUSH_USER=${PUSHOVER_PUSH_USER} \\\n  --env TZ="America/Denver" \\\n  juftin/camply \\\n  camply campsites \\\n      --rec-area 2991 \\\n      --start-date 2023-08-01 \\\n      --end-date 2023-09-01 \\\n      --continuous \\\n      --notifications pushover\n```\n\nThe docker image accepts the following environment variables:\n\n- Pushover Notifications\n    * `PUSHOVER_PUSH_USER`\n- Email Notifications\n    * `EMAIL_TO_ADDRESS`\n    * `EMAIL_USERNAME`\n    * `EMAIL_PASSWORD`\n    * `EMAIL_FROM_ADDRESS` (defaults to "camply@juftin.com")\n    * `EMAIL_SUBJECT_LINE` (defaults to "camply Notification")\n    * `EMAIL_SMTP_SERVER` (defaults to "smtp.gmail.com")\n    * `EMAIL_SMTP_PORT` (defaults to 465)\n- Optional Environment Variables\n    * `LOG_LEVEL` (sets logging level, defaults to "INFO")\n    * `PUSHOVER_PUSH_TOKEN` (Personal Pushover App Token)\n    * `RIDB_API_KEY` (Personal API Key\n      for [Recreation.gov API](https://ridb.recreation.gov/profile))\n    * `TZ` ([TZ Database Name](https://en.wikipedia.org/wiki/List_of_tz_database_time_zones) for\n      logging, defaults to UTC)\n\nAlternatively, if you have already run `camply configure` locally, you can share\nyour [`.camply`](docs/examples/example.camply) file inside the docker container.\n\n```shell\ndocker run \\\n  --name camply-file-share-example \\\n  --env TZ="America/Denver" \\\n  --volume ${HOME}/.camply:/home/camply/.camply \\\n  juftin/camply \\\n  camply campsites \\\n      --provider yellowstone \\\n      --start-date 2023-07-22 \\\n      --end-date 2023-07-27 \\\n      --continuous \\\n      --notifications email\n```\n\nTo manage multiple searches (with different notification preferences) I like to use YAML\nconfiguration files:\n\n```shell\ndocker run -d \\\n  --name camply-email-example \\\n  --env TZ="America/Denver" \\\n  --env EMAIL_TO_ADDRESS=${EMAIL_TO_ADDRESS} \\\n  --env EMAIL_USERNAME=${EMAIL_USERNAME} \\\n  --env EMAIL_PASSWORD=${EMAIL_PASSWORD} \\\n  --volume example_search.yaml:/home/camply/example_search.yaml \\\n  juftin/camply:latest \\\n  camply campsites \\\n      --yaml-config /home/camply/example_search.yaml\n```\n\nA [docker-compose example](docs/examples/docker-compose.yaml) of the above YAML Config is also\navailable.\n\n## Dependencies\n\n`camply` is compatible with any Python version >= `3.6`. Currently, there are eight required\ndependencies:\n\n- [click](https://docs.python-requests.org/en/master/)\n    - The `click` package is used to leverage it\'s simple Command Line Interface\n      API for camply\n- [requests](https://docs.python-requests.org/en/master/)\n    - The `requests` package is used to fetch data from the APIs of Camping Booking Providers.\n- [pandas](https://pandas.pydata.org/)\n    - The `pandas` package is to group and aggregate across large data sets of campsites,\n      campgrounds, and recreation areas.\n- [tenacity](https://tenacity.readthedocs.io/en/latest/)\n    - The `tenacity` package is used for retrying data searches on the underlying campsite APIs.\n      This retrying methodology handles exceptions allowing for API downtime and facilitating\n      exponential backoff.\n- [rich](https://github.com/textualize/rich)\n    - Colorizing the CLI\n- [python-dotenv](https://github.com/theskumar/python-dotenv)\n    - The `python-dotenv` package reads key-value pairs from a `.env` file and can set them as\n      environment variables - this helps with the `.camply` configuration file.\n- [pydantic](https://github.com/samuelcolvin/pydantic)\n    - The `pydantic` package performs data validation against API responses and assists with fancy\n      data containers for `camply` objects.\n- [PyYAML](https://pyyaml.org/)\n    - PyYAML is a YAML parsing library - this helps with the YAML file campsite searches.\n\n<br/>\n\nRecreation data provided by [**Recreation.gov**](https://ridb.recreation.gov/)\n\n___________\n___________\n\n<br/>\n\n[<p align="center" ><img src="https://raw.githubusercontent.com/juftin/juftin/main/static/juftin.png" width="120" height="120"  alt="juftin logo"> </p>](https://github.com/juftin)\n',
    'author': 'Justin Flannery',
    'author_email': 'juftin@juftin.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/juftin/camply',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<4.0',
}


setup(**setup_kwargs)



"""
API Response Objects

These are JSON Responses from APIs
"""

import datetime
from typing import Any, Dict, Iterator, List, Optional, Union

from pydantic import validator

from camply.config.api_config import RecreationBookingConfig
from camply.containers.base_container import (
    CamplyModel,
    RecDotGovAttribute,
    RecDotGovEquipment,
)


class _CampsiteEquipment(CamplyModel):
    EquipmentName: str
    MaxLength: float


class _CampsiteAttribute(CamplyModel):
    AttributeName: str
    AttributeValue: str


class CampsiteResponse(CamplyModel):
    """
    https://ridb.recreation.gov/api/v1/campsites/<CAMPSITE ID>
    """

    CampsiteID: int
    FacilityID: int
    CampsiteName: str
    CampsiteType: str
    TypeOfUse: str
    Loop: str
    CampsiteAccessible: bool
    CampsiteReservable: bool
    CampsiteLongitude: float
    CampsiteLatitude: float
    CreatedDate: datetime.date
    LastUpdatedDate: datetime.date
    PERMITTEDEQUIPMENT: List[_CampsiteEquipment]
    ATTRIBUTES: List[_CampsiteAttribute]


class UnawareDatetime(datetime.datetime):
    """
    Datetime Unaware Timestamp Parsing
    """

    @classmethod
    def __get_validators__(cls) -> Iterator:
        """
        Generate Validators
        """
        yield cls.validate

    @classmethod
    def validate(cls, v: Union[str, datetime.datetime]) -> datetime.datetime:
        """
        Validate Date Strings Into

        Parameters
        ----------
        v: Union[str, datetime.datetime]

        Returns
        -------
        datetime.datetime
        """
        if isinstance(v, str):
            return datetime.datetime.strptime(v, "%Y-%m-%dT%H:%M:%SZ")
        elif isinstance(v, datetime.datetime):
            return v.replace(tzinfo=None)
        else:
            raise ValueError("You Must Provide a Parsable Datetime String or Object")


class _CampsiteAvailabilityCampsiteResponse(CamplyModel):
    """
    https://ridb.recreation.gov/api/v1/campsites/<CAMPSITE ID>
    """

    availabilities: Dict[UnawareDatetime, str] = {}
    loop: str = RecreationBookingConfig.CAMPSITE_LOCATION_LOOP_DEFAULT
    campsite_type: Optional[str]
    max_num_people: int = 1
    min_num_people: int = 1
    type_of_use: Optional[str]
    site: str = RecreationBookingConfig.CAMPSITE_LOCATION_SITE_DEFAULT


class CampsiteAvailabilityResponse(CamplyModel):
    """
    https://ridb.recreation.gov/api/v1/campsites/<CAMPSITE ID>
    """

    campsites: Dict[int, _CampsiteAvailabilityCampsiteResponse]


class _RecAreaAddress(CamplyModel):
    """
    Recreation Area Address Field
    """

    AddressStateCode: str


class RecreationAreaResponse(CamplyModel):
    """
    https://ridb.recreation.gov/api/v1/campsites/<CAMPSITE ID>
    """

    RecAreaID: int
    RecAreaName: str
    RECAREAADDRESS: List[_RecAreaAddress]


class _FacilityAddress(_RecAreaAddress):
    """
    Facility Address aka RecArea Address
    """


class _FacilityRecArea(CamplyModel):
    """
    Recreation Area inside of Facility
    """

    RecAreaID: int
    RecAreaName: str


class _FacilityOrganization(CamplyModel):
    """
    Organization inside of Facility
    """

    OrgName: str
    OrgID: int


class FacilityResponse(CamplyModel):
    """
    /api/v1/facilities/<Facility ID>
    """

    FacilityID: Union[int, str]
    FacilityName: str
    FacilityTypeDescription: str
    Enabled: bool
    Reservable: bool
    FACILITYADDRESS: Optional[List[_FacilityAddress]]
    RECAREA: Optional[List[_FacilityRecArea]]
    ORGANIZATION: Optional[List[_FacilityOrganization]]
    ParentRecAreaID: Optional[int]

    @validator("ParentRecAreaID", pre=True, always=False)
    def validate_parentrecid(cls, val: Any) -> Optional[int]:
        """
        Validate Empty Strings as Null
        """
        if val == "":
            return None
        return val


class _PaginationCountResponse(CamplyModel):
    """
    Pagination Counters
    """

    CURRENT_COUNT: int
    TOTAL_COUNT: int


class _PaginationMetadataResponse(CamplyModel):
    """
    Pagination Metadata
    """

    RESULTS: _PaginationCountResponse


class GenericResponse(CamplyModel):
    """
    Generic Response to Be Paginated
    """

    RECDATA: Any
    METADATA: _PaginationMetadataResponse


class XantPerGuest(CamplyModel):
    """
    PerGuest Objects
    """

    a2: Optional[Union[int, str]]
    b: Optional[Union[int, str]]
    b2: Optional[Union[int, str]]
    m: Optional[Union[int, str]]
    m2: Optional[Union[int, str]]
    r: Optional[Union[int, str]]
    r2: Optional[Union[int, str]]
    s: Optional[Union[int, str]]


class XantRates(CamplyModel):
    """
    Yellowstone Rates Object
    """

    code: str
    title: str
    description: str
    category: str
    minstay: int
    start: datetime.date
    available: Dict[int, int]
    mins: Dict[int, int]
    min: int

    @validator("start", pre=True)
    def parse_datetime(cls, value):
        """
        Parse Poorly Formatted Date Strings
        """
        return datetime.datetime.strptime(value, "%m/%d/%Y").date()


class XantCampgroundDetails(CamplyModel):
    """
    Yellowstone Campground Details OBject
    """

    hotelCode: str
    status: str
    message: str
    min: str
    max: str
    perGuests: Dict[int, XantPerGuest]
    rates: Dict[str, XantRates]
    rates2: Optional[Dict[str, XantRates]]


class XantResortData(CamplyModel):
    """
    Main Yellowstone API Response Wrapper
    """

    availability: Dict[datetime.date, Dict[str, XantCampgroundDetails]]

    @validator("availability", pre=True)
    def parse_datetime(cls, value):
        """
        Parse Poorly Formatted Date Strings
        """
        return {
            datetime.datetime.strptime(x, "%m/%d/%Y").date(): y
            for x, y in value.items()
        }


class RecDotGovCampsite(CamplyModel):
    """
    Recreation.gov Campsite Object
    """

    accessible: bool
    asset_id: int
    asset_type = str
    attributes: List[RecDotGovAttribute]
    average_rating: Optional[int]
    campsite_id: int
    campsite_reserve_type: str
    city: Optional[str]
    country_code: Optional[str]
    fee_templates: Dict[str, Any]
    latitude: Optional[float]
    longitude: Optional[float]
    loop: str
    name: str
    number_of_ratings = int
    org_id: int
    org_name: str
    parent_asset_id: int
    parent_asset_name: str
    parent_asset_type: str
    permitted_equipment: List[RecDotGovEquipment]
    preview_image_url: Optional[str]
    reservable: bool
    state_code: Optional[str]
    type: str
    type_of_use: str


class RecDotGovCampsiteResponse(CamplyModel):
    """
    Parent Response from Campsite Metadata
    """

    campsites: List[RecDotGovCampsite]
    size: int
    spelling_autocorrected: Any
    start: int
    total: int



"""
Camply Command Line Interface
"""

import logging
import sys
from dataclasses import dataclass
from datetime import datetime
from typing import List, Optional, Tuple, Union

import click
from rich import traceback

from camply import __application__, __version__
from camply.config import SearchConfig
from camply.config.logging_config import set_up_logging
from camply.containers import SearchWindow
from camply.providers import RecreationDotGov
from camply.search import CAMPSITE_SEARCH_PROVIDER, SearchYellowstone
from camply.utils import configure_camply, log_camply, make_list, yaml_utils

logging.Logger.camply = log_camply
logger = logging.getLogger(__name__)

DEFAULT_CAMPLY_PROVIDER: str = "RecreationDotGov"


@dataclass
class CamplyContext:
    """
    Context Object Passed Around Application
    """

    debug: Optional[bool] = None
    provider: Optional[str] = None


provider_argument = click.option(
    "--provider",
    show_default=False,
    default=None,
    help="Camping Search Provider. Options available are 'Yellowstone' and "
    "'RecreationDotGov'. Defaults to 'RecreationDotGov', not case-sensitive.",
)
debug_option = click.option(
    "--debug/--no-debug", default=None, help="Enable extra debugging output"
)


def _set_up_debug(debug: Optional[bool] = None) -> None:
    """
    Set up the Camply Debugging Mode
    """
    if debug is None:
        debug = False
    if debug is True:
        set_up_logging(log_level=logging.DEBUG)
        logger.debug("Setting up camply debugging")
        logger.debug("Camply Version: %s", __version__)
        logger.debug("Python Version: %s", sys.version.split(" ")[0])
        logger.debug("Platform: %s", sys.platform)
    traceback.install(show_locals=debug)


@click.group()
@click.version_option(version=__version__, prog_name=__application__)
@provider_argument
@debug_option
@click.pass_context
def camply_command_line(ctx: click.core.Context, provider: str, debug: bool) -> None:
    """
    Welcome to camply, the campsite finder.

    Finding reservations at sold out campgrounds can be
    tough. That's where camply comes in. It searches the APIs of booking services like
    https://recreation.gov (which indexes thousands of campgrounds across the USA) to continuously
    check for cancellations and availabilities to pop up. Once a campsite becomes available,
    camply sends you a notification to book your spot!

    visit the camply documentation at https://github.com/juftin/camply
    """
    set_up_logging(log_level=None if debug is False else logging.INFO)
    logger.camply("camply, the campsite finder ⛺️")  # noqa
    ctx.obj = CamplyContext(debug=debug, provider=provider)
    _set_up_debug(debug=debug)


@camply_command_line.command()
@debug_option
@click.pass_obj
def configure(context: CamplyContext, debug: bool) -> None:
    """
    Set up camply configuration file with an interactive console

    In order to send notifications through camply you must set up some authorization values.
    Whether you need to set up pushover notifications (push notifications on your phone,
    your pushover account can be set up at https://pushover.net) or Email messages, everything
    can be done through the configure command. The end result is a file called .camply in your
    home folder.
    """
    if context.debug is None:
        context.debug = debug
        _set_up_debug(debug=context.debug)
    configure_camply.generate_dot_camply_file()


# Shared Arguments
search_argument = click.option(
    "--search",
    default=None,
    help="Search for Campgrounds or Recreation Areas by search string.",
)
state_argument = click.option("--state", default=None, help="Filter by US state code.")
campsite_id_argument = click.option(
    "--campsite", default=None, multiple=True, help="Add individual Campsites by ID."
)
rec_area_argument = click.option(
    "--rec-area",
    default=None,
    multiple=True,
    help="Add Recreation Areas (comprised of campgrounds) by ID.",
)
campground_argument = click.option(
    "--campground",
    default=None,
    multiple=True,
    help="Add individual Campgrounds by ID.",
)


@camply_command_line.command()
@search_argument
@state_argument
@debug_option
@click.pass_obj
def recreation_areas(
    context: CamplyContext, search: Optional[str], state: Optional[str], debug: bool
) -> None:
    """
    Search for Recreation Areas and list them

    Search for Recreation Areas and their IDs. Recreation Areas are places like
    National Parks and National Forests that can contain one or many campgrounds.
    """
    if context.debug is None:
        context.debug = debug
        _set_up_debug(debug=context.debug)
    if all([search is None, state is None]):
        logger.error(
            "You must add a --search or --state parameter to search "
            "for Recreation Areas."
        )
        exit(1)
    camp_finder = RecreationDotGov()
    params = dict()
    if state is not None:
        params.update(dict(state=state))
    camp_finder.find_recreation_areas(search_string=search, **params)


@camply_command_line.command()
@search_argument
@state_argument
@rec_area_argument
@campground_argument
@campsite_id_argument
@provider_argument
@debug_option
@click.pass_obj
def campgrounds(
    context: CamplyContext,
    debug: bool,
    search: Optional[str] = None,
    state: Optional[str] = None,
    rec_area: Optional[int] = None,
    campground: Optional[int] = None,
    campsite: Optional[int] = None,
    provider: Optional[str] = "RecreationDotGov",
) -> None:
    """
    Search for Campgrounds (inside of Recreation Areas) and list them

    Search for Campgrounds and their IDs. Campgrounds are facilities inside of
    Recreation Areas that contain campsites. Most 'campgrounds' are areas made up of
    multiple campsites, others are facilities like fire towers or cabins that might only
    contain a single 'campsite' to book.
    """
    if context.debug is None:
        context.debug = debug
        _set_up_debug(debug=context.debug)
    if context.provider is None:
        context.provider = provider
    provider = DEFAULT_CAMPLY_PROVIDER if context.provider is None else context.provider
    if provider.lower() == "yellowstone":
        SearchYellowstone.print_campgrounds()
        exit(0)
    if all(
        [
            search is None,
            state is None,
            len(rec_area) == 0,
            len(campground) == 0,
            len(campsite) == 0,
        ]
    ):
        logger.error(
            "You must add a --search, --state, --campground, --campsite, "
            "or --rec-area parameter to search for Campgrounds."
        )
        exit(1)
    camp_finder = RecreationDotGov()
    params = dict()
    if state is not None:
        params.update(dict(state=state))
    camp_finder.find_campgrounds(
        search_string=search,
        rec_area_id=make_list(rec_area, coerce=int),
        campground_id=make_list(campground, coerce=int),
        campsite_id=make_list(campsite, coerce=int),
        **params,
    )


# `campsite` arguments
start_date_argument = click.option(
    "--start-date",
    default=None,
    help="(YYYY-MM-DD) Start of Search window. You will be arriving this day.",
)
end_date_argument = click.option(
    "--end-date",
    default=None,
    help="(YYYY-MM-DD) End of Search window. You will be checking out this day.",
)
weekends_argument = click.option(
    "--weekends",
    is_flag=True,
    show_default=True,
    default=False,
    help="Only search for weekend bookings (Fri/Sat nights).",
)
nights_argument = click.option(
    "--nights",
    show_default=True,
    default=1,
    help="Search for campsite stays with consecutive nights. "
    "Defaults to 1 which returns all campsites found.",
)
continuous_argument = click.option(
    "--continuous",
    is_flag=True,
    show_default=True,
    default=False,
    help="Continuously check for a campsite to become available, and quit once "
    "at least one campsite is found.",
)
polling_interval_argument = click.option(
    "--polling-interval",
    show_default=True,
    default=SearchConfig.RECOMMENDED_POLLING_INTERVAL,
    help="If --continuous is activated, how often to wait in between "
    "checks (in minutes). Defaults to 10, cannot be less than 5.",
)
notifications_argument = click.option(
    "--notifications",
    multiple=True,
    show_default=True,
    default=["silent"],
    help="If --continuous is activated, types of notifications to receive. "
    "Options available are 'email', 'pushover', "
    "'pushbullet', 'telegram', 'twilio', or 'silent'. Defaults to 'silent' - "
    "which just logs messages to console.",
)
notify_first_try_argument = click.option(
    "--notify-first-try",
    is_flag=True,
    show_default=True,
    default=False,
    help="If --continuous is activated, whether to send all "
    "non-silent notifications if more than 5 matching "
    "campsites are found on the first try. Defaults to false which "
    "only sends the first 5.",
)
search_forever_argument = click.option(
    "--search-forever",
    is_flag=True,
    show_default=True,
    default=False,
    help="If --continuous is activated, this method continues to search "
    "after the first availability has been found. The one caveat is "
    "that it will never notify about the same identical campsite for "
    "the same booking date.",
)
yaml_config_argument = click.option(
    "--yaml-config",
    "--yml-config",
    default=None,
    help="Rather than provide arguments to the command line utility, instead "
    "pass a file path to a YAML configuration file. See the documentation "
    "for more information on how to structure your configuration file.",
)
equipment_argument = click.option(
    "--equipment",
    default=None,
    nargs=2,
    multiple=True,
    help="Search for campsites compatible with your camping equipment. "
    "This argument accepts two options, the equipment name and its length "
    "If you don't want to filter based on length provide a length of 0. Accepted "
    "equipment names include `Tent`, `RV`. `Trailer`, `Vehicle` and are "
    "not case-sensitive.",
)
offline_search_argument = click.option(
    "--offline-search",
    is_flag=True,
    show_default=True,
    default=False,
    help="When set to True, the campsite search will both save the results of the "
    "campsites it's found, but also load those campsites before beginning a "
    "search for other campsites.",
)
offline_search_path_argument = click.option(
    "--offline-search-path",
    show_default=True,
    default=None,
    help="When offline search is set to True, this is the name of the file to be saved/loaded. "
    "Campsites can be saved as a serialized pickle file or "
    "a JSON file, depending on the file extension. When not specified, "
    "the filename will default to `camply_campsites.json`",
)


def _get_equipment(equipment: Optional[List[str]]) -> List[Tuple[str, Optional[int]]]:
    """
    Parse Equipment from CLI Args
    """
    equipment_list = []
    for (equipment_name, equipment_length) in equipment:
        try:
            equipment_length = round(float(equipment_length), 0)
            if equipment_length == 0:
                equipment_length = None
        except ValueError:
            equipment_length = None
        equipment_list.append((equipment_name, equipment_length))
    return equipment_list


def _validate_campsites(
    rec_area: Optional[int],
    campground: Optional[int],
    campsite: Optional[int],
    start_date: Optional[str],
    end_date: Optional[str],
    provider: str,
    yaml_config: Optional[str],
    **kwargs,
) -> None:
    """
    Validate the campsites portion of the CLI
    """
    if provider.lower() == "recreationdotgov" and all(
        [
            len(rec_area) == 0,
            len(campground) == 0,
            len(campsite) == 0,
            yaml_config is None,
        ]
    ):
        logger.error(
            "To search for Recreation.gov Campsites you must provide "
            "either the --rec-area, --campground, or --campsite "
            "parameters."
        )
        exit(1)

    if not kwargs.get("notifications") == ["silent"] and not kwargs.get("continuous"):
        logger.error(
            "To receive notifications about campsites you must search "
            "continuously by passing the --continuous option."
        )
        exit(1)

    mandatory_parameters = [start_date, end_date]
    mandatory_string_parameters = ["--start-date", "--end-date"]
    for field in mandatory_parameters:
        if field is None and yaml_config is None:
            logger.error(
                "Campsite searches require the following mandatory search parameters: "
                f"{', '.join(mandatory_string_parameters)}"
            )
            exit(1)


@camply_command_line.command()
@yaml_config_argument
@offline_search_path_argument
@offline_search_argument
@search_forever_argument
@notify_first_try_argument
@notifications_argument
@polling_interval_argument
@continuous_argument
@provider_argument
@equipment_argument
@nights_argument
@weekends_argument
@end_date_argument
@start_date_argument
@campsite_id_argument
@campground_argument
@rec_area_argument
@debug_option
@click.pass_obj
def campsites(
    context: CamplyContext,
    debug: bool,
    rec_area: Optional[int] = None,
    campground: Optional[int] = None,
    campsite: Optional[int] = None,
    start_date: Optional[str] = None,
    end_date: Optional[str] = None,
    weekends: bool = False,
    nights: int = 1,
    provider: str = "RecreationDotGov",
    continuous: bool = False,
    polling_interval: int = SearchConfig.RECOMMENDED_POLLING_INTERVAL,
    notifications: Union[str, List[str]] = "silent",
    notify_first_try: bool = False,
    search_forever: bool = False,
    yaml_config: Optional[str] = None,
    equipment: Optional[List[str]] = None,
    offline_search: bool = False,
    offline_search_path: Optional[str] = None,
) -> None:
    """
    Find available Campsites using search criteria

    Search for a campsite within camply. Campsites are returned based on the search criteria
    provided. Campsites contain properties like booking date, site type (tent, RV, cabin, etc),
    capacity, price, and a link to make the booking. Required parameters include
    `--start-date`, `--end-date`, `--rec-area` / `--campground`. Constant searching
    functionality can be enabled with  `--continuous` and notifications can be enabled using
    `--notifications`.
    """
    if context.debug is None:
        context.debug = debug
        _set_up_debug(debug=context.debug)
    if context.provider is None:
        context.provider = provider
    provider = DEFAULT_CAMPLY_PROVIDER if context.provider is None else context.provider
    notifications = make_list(notifications)
    _validate_campsites(
        rec_area=rec_area,
        campground=campground,
        campsite=campsite,
        start_date=start_date,
        end_date=end_date,
        weekends=weekends,
        nights=nights,
        provider=provider,
        continuous=continuous,
        polling_interval=polling_interval,
        notifications=notifications,
        notify_first_try=notify_first_try,
        search_forever=search_forever,
        yaml_config=yaml_config,
    )
    if yaml_config is not None:
        provider, provider_kwargs, search_kwargs = yaml_utils.yaml_file_to_arguments(
            file_path=yaml_config
        )
    else:
        provider = provider.lower()
        search_window = SearchWindow(
            start_date=datetime.strptime(start_date, "%Y-%m-%d"),
            end_date=datetime.strptime(end_date, "%Y-%m-%d"),
        )
        provider_kwargs = dict(
            search_window=search_window,
            recreation_area=make_list(rec_area),
            campgrounds=make_list(campground),
            campsites=make_list(campsite),
            weekends_only=weekends,
            nights=int(nights),
            equipment=make_list(equipment),
            offline_search=offline_search,
            offline_search_path=offline_search_path,
        )
        search_kwargs = dict(
            log=True,
            verbose=True,
            continuous=continuous,
            polling_interval=float(polling_interval),
            notify_first_try=notify_first_try,
            notification_provider=notifications,
            search_forever=search_forever,
        )
    provider_class = {
        key.lower(): value for key, value in CAMPSITE_SEARCH_PROVIDER.items()
    }[provider.lower()]
    camping_finder = provider_class(**provider_kwargs)
    camping_finder.get_matching_campsites(**search_kwargs)


def cli():
    """
    Camply Command Line Utility Wrapper
    """
    try:
        camply_command_line()
    except KeyboardInterrupt:
        logger.debug("Handling Exit Request")
    finally:
        logger.camply("Exiting camply 👋")  # noqa


if __name__ == "__main__":
    cli()
